@@ -26,7 +26,7 @@ using namespace std;
 #include "common/strtol.h"
 
 #include "mon/MonMap.h"
-#include "mds/MDS.h"
+#include "mds/MDSDaemon.h"
 
 #include "msg/Messenger.h"
 
@@ -78,7 +78,7 @@ static int parse_rank(const char *opt_name, const std::string &val)
 
 
 
-MDS *mds = NULL;
+MDSDaemon *mds = NULL;
 
 
 static void handle_mds_signal(int signum)
@@ -145,18 +145,19 @@ int main(int argc, const char **argv)
     usage();
   }
 
-  if (g_conf->name.get_id().empty() || (g_conf->name.get_id()[0] >= '0' && g_conf->name.get_id()[0] <= '9')) {
+  if (g_conf->name.get_id().empty() ||
+      (g_conf->name.get_id()[0] >= '0' && g_conf->name.get_id()[0] <= '9')) {
     derr << "deprecation warning: MDS id '" << g_conf->name
       << "' is invalid and will be forbidden in a future version.  "
       "MDS names may not start with a numeric digit." << dendl;
   }
 
-  Messenger *messenger = Messenger::create(g_ceph_context, g_conf->ms_type,
-					   entity_name_t::MDS(-1), "mds",
-					   getpid());
-  messenger->set_cluster_protocol(CEPH_MDS_PROTOCOL);
+  Messenger *msgr = Messenger::create(g_ceph_context, g_conf->ms_type,
+				      entity_name_t::MDS(-1), "mds",
+				      getpid());
+  msgr->set_cluster_protocol(CEPH_MDS_PROTOCOL);
 
-  cout << "starting " << g_conf->name << " at " << messenger->get_myaddr()
+  cout << "starting " << g_conf->name << " at " << msgr->get_myaddr()
        << std::endl;
   uint64_t supported =
     CEPH_FEATURE_UID |
@@ -169,23 +170,24 @@ int main(int argc, const char **argv)
     CEPH_FEATURE_MDS_QUOTA;
   uint64_t required =
     CEPH_FEATURE_OSDREPLYMUX;
-  messenger->set_default_policy(Messenger::Policy::lossy_client(supported, required));
-  messenger->set_policy(entity_name_t::TYPE_MON,
-			Messenger::Policy::lossy_client(supported,
-							CEPH_FEATURE_UID |
-							CEPH_FEATURE_PGID64));
-  messenger->set_policy(entity_name_t::TYPE_MDS,
-			Messenger::Policy::lossless_peer(supported,
-							 CEPH_FEATURE_UID));
-  messenger->set_policy(entity_name_t::TYPE_CLIENT,
-			Messenger::Policy::stateful_server(supported, 0));
 
-  int r = messenger->bind(g_conf->public_addr);
+  msgr->set_default_policy(Messenger::Policy::lossy_client(supported, required));
+  msgr->set_policy(entity_name_t::TYPE_MON,
+                   Messenger::Policy::lossy_client(supported,
+                                                   CEPH_FEATURE_UID |
+                                                   CEPH_FEATURE_PGID64));
+  msgr->set_policy(entity_name_t::TYPE_MDS,
+                   Messenger::Policy::lossless_peer(supported,
+                                                    CEPH_FEATURE_UID));
+  msgr->set_policy(entity_name_t::TYPE_CLIENT,
+                   Messenger::Policy::stateful_server(supported, 0));
+
+  int r = msgr->bind(g_conf->public_addr);
   if (r < 0)
     exit(1);
 
   if (shadow != MDSMap::STATE_ONESHOT_REPLAY)
-    global_init_daemonize(g_ceph_context, 0);
+    global_init_daemonize(g_ceph_context);
   common_init_finish(g_ceph_context);
 
   // get monmap
@@ -194,16 +196,16 @@ int main(int argc, const char **argv)
     return -1;
   global_init_chdir(g_ceph_context);
 
-  messenger->start();
+  msgr->start();
 
   // start mds
-  mds = new MDS(g_conf->name.get_id().c_str(), messenger, &mc);
+  mds = new MDSDaemon(g_conf->name.get_id().c_str(), msgr, &mc);
 
   // in case we have to respawn...
   mds->orig_argc = argc;
   mds->orig_argv = argv;
 
-  if (shadow)
+  if (shadow != MDSMap::STATE_NULL)
     r = mds->init(shadow);
   else
     r = mds->init();
@@ -219,7 +221,7 @@ int main(int argc, const char **argv)
   if (g_conf->inject_early_sigterm)
     kill(getpid(), SIGTERM);
 
-  messenger->wait();
+  msgr->wait();
 
   unregister_async_signal_handler(SIGHUP, sighup_handler);
   unregister_async_signal_handler(SIGINT, handle_mds_signal);
@@ -236,8 +238,10 @@ int main(int argc, const char **argv)
 
   // only delete if it was a clean shutdown (to aid memory leak
   // detection, etc.).  don't bother if it was a suicide.
-  if (mds->is_stopped())
+  if (mds->is_clean_shutdown()) {
     delete mds;
+    delete msgr;
+  }
 
   g_ceph_context->put();
 
