@@ -1,4 +1,5 @@
 
+#include "acconfig.h"
 #include "include/types.h"
 #include "MemoryModel.h"
 #include "common/config.h"
@@ -92,7 +93,7 @@ void MemoryModel::_sample(snap *psnap)
   psnap->heap = heap >> 10;
 
   // ...
-#if defined(__linux__)
+#if defined(HAVE_MALLINFO)
   struct mallinfo mi = mallinfo();
   
   psnap->malloc = mi.uordblks >> 10;
