@@ -11,6 +11,9 @@
 #elif defined(__FreeBSD__)
 #include <sys/cdefs.h>
 #define	__GNUC_PREREQ(minor, major)	__GNUC_PREREQ__(minor, major)
+#elif defined(__sun)
+#include "include/compat.h"
+#include <assert.h>
 #endif
 
 #ifdef __CEPH__
