@@ -2,7 +2,7 @@
 // vim: ts=8 sw=2 smarttab
 #include "librbd/AsyncTrimRequest.h"
 #include "librbd/AsyncObjectThrottle.h"
-#include "librbd/AioRequest.h"
+#include "librbd/AioObjectRequest.h"
 #include "librbd/ImageCtx.h"
 #include "librbd/ImageWatcher.h"
 #include "librbd/internal.h"
@@ -41,8 +41,8 @@ public:
     string oid = m_image_ctx.get_object_name(m_object_no);
     ldout(m_image_ctx.cct, 10) << "removing (with copyup) " << oid << dendl;
 
-    AbstractWrite *req = new AioTrim(&m_image_ctx, oid, m_object_no, m_snapc,
-                                     this);
+    AioObjectRequest *req = new AioObjectTrim(&m_image_ctx, oid, m_object_no,
+                                              m_snapc, this);
     req->send();
     return 0;
   }
@@ -340,13 +340,13 @@ void AsyncTrimRequest::send_clean_boundary() {
     ldout(cct, 20) << " ex " << *p << dendl;
     Context *req_comp = new C_ContextCompletion(*completion);
 
-    AbstractWrite *req;
+    AioObjectRequest *req;
     if (p->offset == 0) {
-      req = new AioTrim(&m_image_ctx, p->oid.name, p->objectno, snapc,
-                        req_comp);
+      req = new AioObjectTrim(&m_image_ctx, p->oid.name, p->objectno, snapc,
+                              req_comp);
     } else {
-      req = new AioTruncate(&m_image_ctx, p->oid.name, p->objectno,
-                            p->offset, snapc, req_comp);
+      req = new AioObjectTruncate(&m_image_ctx, p->oid.name, p->objectno,
+                                  p->offset, snapc, req_comp);
     }
     req->send();
   }
