@@ -20,7 +20,7 @@ namespace librbd {
                   vector<pair<uint64_t,uint64_t> >& image_extents);
     ~CopyupRequest();
 
-    void append_request(AioRequest *req);
+    void append_request(AioObjectRequest *req);
 
     void send();
     void queue_send();
@@ -65,7 +65,7 @@ namespace librbd {
     vector<pair<uint64_t,uint64_t> > m_image_extents;
     State m_state;
     ceph::bufferlist m_copyup_data;
-    vector<AioRequest *> m_pending_requests;
+    vector<AioObjectRequest *> m_pending_requests;
     atomic_t m_pending_copyups;
 
     AsyncOperation m_async_op;
