@@ -34,11 +34,12 @@ ImageWatcher::ImageWatcher(ImageCtx &image_ctx)
   : m_image_ctx(image_ctx),
     m_watch_lock(unique_lock_name("librbd::ImageWatcher::m_watch_lock", this)),
     m_watch_ctx(*this), m_watch_handle(0),
-    m_watch_state(WATCH_STATE_UNREGISTERED),
+    m_watch_state(WATCH_STATE_UNREGISTERED), m_lock_supported(false),
     m_lock_owner_state(LOCK_OWNER_STATE_NOT_LOCKED),
+    m_listeners_lock(unique_lock_name("librbd::ImageWatcher::m_listeners_lock", this)),
+    m_listeners_in_use(false),
     m_task_finisher(new TaskFinisher<Task>(*m_image_ctx.cct)),
     m_async_request_lock(unique_lock_name("librbd::ImageWatcher::m_async_request_lock", this)),
-    m_aio_request_lock(unique_lock_name("librbd::ImageWatcher::m_aio_request_lock", this)),
     m_owner_client_id_lock(unique_lock_name("librbd::ImageWatcher::m_owner_client_id_lock", this))
 {
 }
@@ -74,6 +75,20 @@ bool ImageWatcher::is_lock_owner() const {
           m_lock_owner_state == LOCK_OWNER_STATE_RELEASING);
 }
 
+void ImageWatcher::register_listener(Listener *listener) {
+  Mutex::Locker listeners_locker(m_listeners_lock);
+  m_listeners.push_back(listener);
+}
+
+void ImageWatcher::unregister_listener(Listener *listener) {
+  // TODO CoW listener list
+  Mutex::Locker listeners_locker(m_listeners_lock);
+  while (m_listeners_in_use) {
+    m_listeners_cond.Wait(m_listeners_lock);
+  }
+  m_listeners.remove(listener);
+}
+
 int ImageWatcher::register_watch() {
   ldout(m_image_ctx.cct, 10) << this << " registering image watcher" << dendl;
 
@@ -93,11 +108,6 @@ int ImageWatcher::register_watch() {
 int ImageWatcher::unregister_watch() {
   ldout(m_image_ctx.cct, 10) << this << " unregistering image watcher" << dendl;
 
-  {
-    Mutex::Locker l(m_aio_request_lock);
-    assert(m_aio_requests.empty());
-  }
-
   cancel_async_requests();
   m_task_finisher->cancel_all();
 
@@ -115,9 +125,44 @@ int ImageWatcher::unregister_watch() {
   return r;
 }
 
+int ImageWatcher::refresh() {
+  assert(m_image_ctx.owner_lock.is_locked());
+
+  bool lock_support_changed = false;
+  {
+    RWLock::WLocker watch_locker(m_watch_lock);
+    if (m_lock_supported != is_lock_supported()) {
+      m_lock_supported = is_lock_supported();
+      lock_support_changed = true;
+    }
+  }
+
+  int r = 0;
+  if (lock_support_changed) {
+    if (is_lock_supported()) {
+      // image opened, exclusive lock dynamically enabled, or now HEAD
+      notify_listeners_updated_lock(LOCK_UPDATE_STATE_RELEASING);
+      notify_listeners_updated_lock(LOCK_UPDATE_STATE_UNLOCKED);
+    } else if (!is_lock_supported()) {
+      if (is_lock_owner()) {
+        // exclusive lock dynamically disabled or now snapshot
+        m_image_ctx.owner_lock.put_read();
+        {
+          RWLock::WLocker owner_locker(m_image_ctx.owner_lock);
+          r = release_lock();
+        }
+        m_image_ctx.owner_lock.get_read();
+      }
+      notify_listeners_updated_lock(LOCK_UPDATE_STATE_NOT_SUPPORTED);
+    }
+  }
+  return r;
+}
+
 int ImageWatcher::try_lock() {
   assert(m_image_ctx.owner_lock.is_wlocked());
   assert(m_lock_owner_state == LOCK_OWNER_STATE_NOT_LOCKED);
+  assert(is_lock_supported());
 
   while (true) {
     int r = lock();
@@ -182,33 +227,8 @@ int ImageWatcher::try_lock() {
   return 0;
 }
 
-void ImageWatcher::request_lock(
-    const boost::function<void(AioCompletion*)>& restart_op, AioCompletion* c) {
-  assert(m_image_ctx.owner_lock.is_locked());
-  assert(m_lock_owner_state == LOCK_OWNER_STATE_NOT_LOCKED);
-
-  {
-    Mutex::Locker l(m_aio_request_lock);
-    bool request_pending = !m_aio_requests.empty();
-    ldout(m_image_ctx.cct, 15) << this << " queuing aio request: " << c
-			       << dendl;
-
-    c->get();
-    m_aio_requests.push_back(std::make_pair(restart_op, c));
-    if (request_pending) {
-      return;
-    }
-  }
-
-  RWLock::RLocker l(m_watch_lock);
-  if (m_watch_state == WATCH_STATE_REGISTERED) {
-    ldout(m_image_ctx.cct, 15) << this << " requesting exclusive lock" << dendl;
-
-    // run notify request in finisher to avoid blocking aio path
-    FunctionContext *ctx = new FunctionContext(
-      boost::bind(&ImageWatcher::notify_request_lock, this));
-    m_task_finisher->queue(TASK_CODE_REQUEST_LOCK, ctx);
-  }
+void ImageWatcher::request_lock() {
+  schedule_request_lock(false);
 }
 
 bool ImageWatcher::try_request_lock() {
@@ -291,6 +311,9 @@ int ImageWatcher::get_lock_owner_info(entity_name_t *locker, std::string *cookie
 }
 
 int ImageWatcher::lock() {
+  assert(m_image_ctx.owner_lock.is_wlocked());
+  assert(m_lock_owner_state == LOCK_OWNER_STATE_NOT_LOCKED);
+
   int r = rados::cls::lock::lock(&m_image_ctx.md_ctx, m_image_ctx.header_oid,
 				 RBD_LOCK_NAME, LOCK_EXCLUSIVE,
 				 encode_lock_cookie(), WATCHER_LOCK_TAG, "",
@@ -318,37 +341,16 @@ int ImageWatcher::lock() {
     m_image_ctx.object_map.refresh(CEPH_NOSNAP);
   }
 
-  bufferlist bl;
-  ::encode(NotifyMessage(AcquiredLockPayload(get_client_id())), bl);
-
   // send the notification when we aren't holding locks
   FunctionContext *ctx = new FunctionContext(
-    boost::bind(&IoCtx::notify2, &m_image_ctx.md_ctx, m_image_ctx.header_oid,
-		bl, NOTIFY_TIMEOUT, reinterpret_cast<bufferlist *>(0)));
+    boost::bind(&ImageWatcher::notify_acquired_lock, this));
   m_task_finisher->queue(TASK_CODE_ACQUIRED_LOCK, ctx);
   return 0;
 }
 
-void ImageWatcher::prepare_unlock() {
-  assert(m_image_ctx.owner_lock.is_wlocked());
-  if (m_lock_owner_state == LOCK_OWNER_STATE_LOCKED) {
-    m_lock_owner_state = LOCK_OWNER_STATE_RELEASING;
-  }
-}
-
-void ImageWatcher::cancel_unlock() {
-  assert(m_image_ctx.owner_lock.is_wlocked());
-  if (m_lock_owner_state == LOCK_OWNER_STATE_RELEASING) {
-    m_lock_owner_state = LOCK_OWNER_STATE_LOCKED;
-  }
-}
-
 int ImageWatcher::unlock()
 {
   assert(m_image_ctx.owner_lock.is_wlocked());
-  if (m_lock_owner_state == LOCK_OWNER_STATE_NOT_LOCKED) {
-    return 0;
-  }
 
   ldout(m_image_ctx.cct, 10) << this << " releasing exclusive lock" << dendl;
   m_lock_owner_state = LOCK_OWNER_STATE_NOT_LOCKED;
@@ -364,8 +366,10 @@ int ImageWatcher::unlock()
     m_image_ctx.object_map.unlock();
   }
 
-  Mutex::Locker l(m_owner_client_id_lock);
-  set_owner_client_id(ClientId());
+  {
+    Mutex::Locker l(m_owner_client_id_lock);
+    set_owner_client_id(ClientId());
+  }
 
   FunctionContext *ctx = new FunctionContext(
     boost::bind(&ImageWatcher::notify_released_lock, this));
@@ -373,32 +377,66 @@ int ImageWatcher::unlock()
   return 0;
 }
 
-bool ImageWatcher::release_lock()
+int ImageWatcher::release_lock()
 {
   assert(m_image_ctx.owner_lock.is_wlocked());
-  ldout(m_image_ctx.cct, 10) << this << " releasing exclusive lock by request"
-                             << dendl;
-  if (!is_lock_owner()) {
-    return false;
+
+  CephContext *cct = m_image_ctx.cct;
+  ldout(cct, 10) << this << " releasing exclusive lock by request" << dendl;
+  if (m_lock_owner_state != LOCK_OWNER_STATE_LOCKED) {
+    return 0;
   }
-  prepare_unlock();
+
+  m_lock_owner_state = LOCK_OWNER_STATE_RELEASING;
   m_image_ctx.owner_lock.put_write();
+
+  // ensure all maint operations are canceled
   m_image_ctx.cancel_async_requests();
   m_image_ctx.flush_async_operations();
 
+  int r;
   {
     RWLock::RLocker owner_locker(m_image_ctx.owner_lock);
+
+    // alert listeners that all incoming IO needs to be stopped since the
+    // lock is being released
+    notify_listeners_updated_lock(LOCK_UPDATE_STATE_RELEASING);
+
     RWLock::WLocker md_locker(m_image_ctx.md_lock);
-    librbd::_flush(&m_image_ctx);
+    r = m_image_ctx.flush();
+    if (r < 0) {
+      lderr(cct) << this << " failed to flush: " << cpp_strerror(r) << dendl;
+      goto err_cancel_unlock;
+    }
   }
 
   m_image_ctx.owner_lock.get_write();
-  if (!is_lock_owner()) {
-    return false;
+  assert(m_lock_owner_state == LOCK_OWNER_STATE_RELEASING);
+  r = unlock();
+
+  // notify listeners of the change w/ owner read locked
+  m_image_ctx.owner_lock.put_write();
+  {
+    RWLock::RLocker owner_lock(m_image_ctx.owner_lock);
+    if (m_lock_owner_state == LOCK_OWNER_STATE_NOT_LOCKED) {
+      notify_listeners_updated_lock(LOCK_UPDATE_STATE_UNLOCKED);
+    }
+  }
+  m_image_ctx.owner_lock.get_write();
+
+  if (r < 0) {
+    lderr(cct) << this << " failed to unlock: " << cpp_strerror(r) << dendl;
+    return r;
   }
 
-  unlock();
-  return true;
+  return 0;
+
+err_cancel_unlock:
+  m_image_ctx.owner_lock.get_write();
+  if (m_lock_owner_state == LOCK_OWNER_STATE_RELEASING) {
+    m_lock_owner_state = LOCK_OWNER_STATE_LOCKED;
+  }
+  return r;
 }
 
 void ImageWatcher::assert_header_locked(librados::ObjectWriteOperation *op) {
@@ -529,6 +567,21 @@ int ImageWatcher::notify_rebuild_object_map(uint64_t request_id,
   return notify_async_request(async_request_id, bl, prog_ctx);
 }
 
+void ImageWatcher::notify_lock_state() {
+  RWLock::RLocker owner_locker(m_image_ctx.owner_lock);
+  if (m_lock_owner_state == LOCK_OWNER_STATE_LOCKED) {
+    // re-send the acquired lock notification so that peers know they can now
+    // request the lock
+    ldout(m_image_ctx.cct, 10) << this << " notify lock state" << dendl;
+
+    bufferlist bl;
+    ::encode(NotifyMessage(AcquiredLockPayload(get_client_id())), bl);
+
+    m_image_ctx.md_ctx.notify2(m_image_ctx.header_oid, bl, NOTIFY_TIMEOUT,
+                               NULL);
+  }
+}
+
 void ImageWatcher::notify_header_update(librados::IoCtx &io_ctx,
 				        const std::string &oid)
 {
@@ -556,37 +609,6 @@ bool ImageWatcher::decode_lock_cookie(const std::string &tag,
   return true;
 }
 
-void ImageWatcher::schedule_retry_aio_requests(bool use_timer) {
-  m_task_finisher->cancel(TASK_CODE_REQUEST_LOCK);
-  Context *ctx = new FunctionContext(boost::bind(
-    &ImageWatcher::retry_aio_requests, this));
-  if (use_timer) {
-    m_task_finisher->add_event_after(TASK_CODE_RETRY_AIO_REQUESTS,
-                                     RETRY_DELAY_SECONDS, ctx);
-  } else {
-    m_task_finisher->queue(TASK_CODE_RETRY_AIO_REQUESTS, ctx);
-  }
-}
-
-void ImageWatcher::retry_aio_requests() {
-  m_task_finisher->cancel(TASK_CODE_RETRY_AIO_REQUESTS);
-  std::vector<AioRequest> lock_request_restarts;
-  {
-    Mutex::Locker l(m_aio_request_lock);
-    lock_request_restarts.swap(m_aio_requests);
-  }
-
-  ldout(m_image_ctx.cct, 15) << this << " retrying pending aio requests"
-                             << dendl;
-  for (std::vector<AioRequest>::iterator iter = lock_request_restarts.begin();
-       iter != lock_request_restarts.end(); ++iter) {
-    ldout(m_image_ctx.cct, 20) << this << " retrying aio request: "
-                               << iter->second << dendl;
-    iter->first(iter->second);
-    iter->second->put();
-  }
-}
-
 void ImageWatcher::schedule_cancel_async_requests() {
   FunctionContext *ctx = new FunctionContext(
     boost::bind(&ImageWatcher::cancel_async_requests, this));
@@ -615,6 +637,21 @@ ClientId ImageWatcher::get_client_id() {
   return ClientId(m_image_ctx.md_ctx.get_instance_id(), m_watch_handle);
 }
 
+void ImageWatcher::notify_acquired_lock() {
+  ldout(m_image_ctx.cct, 10) << this << " notify acquired lock" << dendl;
+
+  RWLock::RLocker owner_locker(m_image_ctx.owner_lock);
+  if (m_lock_owner_state != LOCK_OWNER_STATE_LOCKED) {
+    return;
+  }
+
+  notify_listeners_updated_lock(LOCK_UPDATE_STATE_LOCKED);
+
+  bufferlist bl;
+  ::encode(NotifyMessage(AcquiredLockPayload(get_client_id())), bl);
+  m_image_ctx.md_ctx.notify2(m_image_ctx.header_oid, bl, NOTIFY_TIMEOUT, NULL);
+}
+
 void ImageWatcher::notify_release_lock() {
   RWLock::WLocker owner_locker(m_image_ctx.owner_lock);
   release_lock();
@@ -627,14 +664,33 @@ void ImageWatcher::notify_released_lock() {
   m_image_ctx.md_ctx.notify2(m_image_ctx.header_oid, bl, NOTIFY_TIMEOUT, NULL);
 }
 
+void ImageWatcher::schedule_request_lock(bool use_timer, int timer_delay) {
+  assert(m_image_ctx.owner_lock.is_locked());
+  assert(m_lock_owner_state == LOCK_OWNER_STATE_NOT_LOCKED);
+
+  RWLock::RLocker watch_locker(m_watch_lock);
+  if (m_watch_state == WATCH_STATE_REGISTERED) {
+    ldout(m_image_ctx.cct, 15) << this << " requesting exclusive lock" << dendl;
+
+    FunctionContext *ctx = new FunctionContext(
+      boost::bind(&ImageWatcher::notify_request_lock, this));
+    if (use_timer) {
+      if (timer_delay < 0) {
+        timer_delay = RETRY_DELAY_SECONDS;
+      }
+      m_task_finisher->add_event_after(TASK_CODE_REQUEST_LOCK, timer_delay,
+                                       ctx);
+    } else {
+      m_task_finisher->queue(TASK_CODE_REQUEST_LOCK, ctx);
+    }
+  }
+}
+
 void ImageWatcher::notify_request_lock() {
   ldout(m_image_ctx.cct, 10) << this << " notify request lock" << dendl;
-  m_task_finisher->cancel(TASK_CODE_RETRY_AIO_REQUESTS);
 
-  m_image_ctx.owner_lock.get_read();
+  RWLock::RLocker owner_locker(m_image_ctx.owner_lock);
   if (try_request_lock()) {
-    m_image_ctx.owner_lock.put_read();
-    retry_aio_requests();
     return;
   }
 
@@ -642,23 +698,20 @@ void ImageWatcher::notify_request_lock() {
   ::encode(NotifyMessage(RequestLockPayload(get_client_id())), bl);
 
   int r = notify_lock_owner(bl);
-  m_image_ctx.owner_lock.put_read();
-
   if (r == -ETIMEDOUT) {
-    ldout(m_image_ctx.cct, 5) << this << "timed out requesting lock: retrying"
+    ldout(m_image_ctx.cct, 5) << this << " timed out requesting lock: retrying"
                               << dendl;
-    retry_aio_requests();
+    schedule_request_lock(false);
   } else if (r < 0) {
     lderr(m_image_ctx.cct) << this << " error requesting lock: "
                            << cpp_strerror(r) << dendl;
-    schedule_retry_aio_requests(true);
+    schedule_request_lock(true);
   } else {
     // lock owner acked -- but resend if we don't see them release the lock
     int retry_timeout = m_image_ctx.cct->_conf->client_notify_timeout;
-    FunctionContext *ctx = new FunctionContext(
-      boost::bind(&ImageWatcher::notify_request_lock, this));
-    m_task_finisher->add_event_after(TASK_CODE_REQUEST_LOCK,
-                                     retry_timeout, ctx);
+    ldout(m_image_ctx.cct, 15) << this << " will retry in " << retry_timeout
+                               << " seconds" << dendl;
+    schedule_request_lock(true, retry_timeout);
   }
 }
 
@@ -813,40 +866,48 @@ void ImageWatcher::handle_payload(const AcquiredLockPayload &payload,
                                   bufferlist *out) {
   ldout(m_image_ctx.cct, 10) << this << " image exclusively locked announcement"
                              << dendl;
+
+  bool cancel_async_requests = true;
   if (payload.client_id.is_valid()) {
     Mutex::Locker l(m_owner_client_id_lock);
     if (payload.client_id == m_owner_client_id) {
-      // we already know that the remote client is the owner
-      return;
+      cancel_async_requests = false;
     }
     set_owner_client_id(payload.client_id);
   }
 
-  RWLock::RLocker l(m_image_ctx.owner_lock);
+  RWLock::RLocker owner_locker(m_image_ctx.owner_lock);
   if (m_lock_owner_state == LOCK_OWNER_STATE_NOT_LOCKED) {
-    schedule_cancel_async_requests();
-    schedule_retry_aio_requests(false);
+    if (cancel_async_requests) {
+      schedule_cancel_async_requests();
+    }
+    notify_listeners_updated_lock(LOCK_UPDATE_STATE_NOTIFICATION);
   }
 }
 
 void ImageWatcher::handle_payload(const ReleasedLockPayload &payload,
                                   bufferlist *out) {
   ldout(m_image_ctx.cct, 10) << this << " exclusive lock released" << dendl;
+
+  bool cancel_async_requests = true;
   if (payload.client_id.is_valid()) {
     Mutex::Locker l(m_owner_client_id_lock);
     if (payload.client_id != m_owner_client_id) {
       ldout(m_image_ctx.cct, 10) << this << " unexpected owner: "
                                  << payload.client_id << " != "
                                  << m_owner_client_id << dendl;
-      return;
+      cancel_async_requests = false;
+    } else {
+      set_owner_client_id(ClientId());
     }
-    set_owner_client_id(ClientId());
   }
 
-  RWLock::RLocker l(m_image_ctx.owner_lock);
+  RWLock::RLocker owner_locker(m_image_ctx.owner_lock);
   if (m_lock_owner_state == LOCK_OWNER_STATE_NOT_LOCKED) {
-    schedule_cancel_async_requests();
-    schedule_retry_aio_requests(false);
+    if (cancel_async_requests) {
+      schedule_cancel_async_requests();
+    }
+    notify_listeners_updated_lock(LOCK_UPDATE_STATE_NOTIFICATION);
   }
 }
 
@@ -869,11 +930,25 @@ void ImageWatcher::handle_payload(const RequestLockPayload &payload,
       }
     }
 
-    ldout(m_image_ctx.cct, 10) << this << " queuing release of exclusive lock"
-                               << dendl;
-    FunctionContext *ctx = new FunctionContext(
-      boost::bind(&ImageWatcher::notify_release_lock, this));
-    m_task_finisher->queue(TASK_CODE_RELEASING_LOCK, ctx);
+    bool release_permitted = true;
+    {
+      Mutex::Locker listeners_locker(m_listeners_lock);
+      for (Listeners::iterator it = m_listeners.begin();
+           it != m_listeners.end(); ++it) {
+        if (!(*it)->handle_requested_lock()) {
+          release_permitted = false;
+          break;
+        }
+      }
+    }
+
+    if (release_permitted) {
+      ldout(m_image_ctx.cct, 10) << this << " queuing release of exclusive lock"
+                                 << dendl;
+      FunctionContext *ctx = new FunctionContext(
+        boost::bind(&ImageWatcher::notify_release_lock, this));
+      m_task_finisher->queue(TASK_CODE_RELEASING_LOCK, ctx);
+    }
   }
 }
 
@@ -1078,53 +1153,53 @@ void ImageWatcher::acknowledge_notify(uint64_t notify_id, uint64_t handle,
 void ImageWatcher::reregister_watch() {
   ldout(m_image_ctx.cct, 10) << this << " re-registering image watch" << dendl;
 
+  RWLock::WLocker l(m_image_ctx.owner_lock);
+  bool was_lock_owner = false;
+  if (m_lock_owner_state == LOCK_OWNER_STATE_LOCKED) {
+    // ensure all async requests are canceled and IO is flushed
+    was_lock_owner = release_lock();
+  }
+
+  int r;
   {
-    RWLock::WLocker l(m_image_ctx.owner_lock);
-    bool was_lock_owner = false;
-    if (m_lock_owner_state == LOCK_OWNER_STATE_LOCKED) {
-      // ensure all async requests are canceled and IO is flushed
-      was_lock_owner = release_lock();
+    RWLock::WLocker l(m_watch_lock);
+    if (m_watch_state != WATCH_STATE_ERROR) {
+      return;
     }
 
-    int r;
-    {
-      RWLock::WLocker l(m_watch_lock);
-      if (m_watch_state != WATCH_STATE_ERROR) {
-	return;
+    r = m_image_ctx.md_ctx.watch2(m_image_ctx.header_oid,
+                                  &m_watch_handle, &m_watch_ctx);
+    if (r < 0) {
+      lderr(m_image_ctx.cct) << this << " failed to re-register image watch: "
+                             << cpp_strerror(r) << dendl;
+      if (r != -ESHUTDOWN) {
+        FunctionContext *ctx = new FunctionContext(boost::bind(
+          &ImageWatcher::reregister_watch, this));
+        m_task_finisher->add_event_after(TASK_CODE_REREGISTER_WATCH,
+                                         RETRY_DELAY_SECONDS, ctx);
       }
-
-      r = m_image_ctx.md_ctx.watch2(m_image_ctx.header_oid,
-                                    &m_watch_handle, &m_watch_ctx);
-      if (r < 0) {
-        lderr(m_image_ctx.cct) << this << " failed to re-register image watch: "
-                               << cpp_strerror(r) << dendl;
-	if (r != -ESHUTDOWN) {
-	  FunctionContext *ctx = new FunctionContext(boost::bind(
-	    &ImageWatcher::reregister_watch, this));
-	  m_task_finisher->add_event_after(TASK_CODE_REREGISTER_WATCH,
-                                           RETRY_DELAY_SECONDS, ctx);
-	}
-        return;
-      }
-
-      m_watch_state = WATCH_STATE_REGISTERED;
+      return;
     }
-    handle_payload(HeaderUpdatePayload(), NULL);
 
-    if (was_lock_owner) {
-      r = try_lock();
-      if (r == -EBUSY) {
-        ldout(m_image_ctx.cct, 5) << this << "lost image lock while "
-                                  << "re-registering image watch" << dendl;
-      } else if (r < 0) {
-        lderr(m_image_ctx.cct) << this
-                               << "failed to lock image while re-registering "
-                               << "image watch" << cpp_strerror(r) << dendl;
-      }
+    m_watch_state = WATCH_STATE_REGISTERED;
+  }
+  handle_payload(HeaderUpdatePayload(), NULL);
+
+  if (was_lock_owner) {
+    r = try_lock();
+    if (r == -EBUSY) {
+      ldout(m_image_ctx.cct, 5) << this << "lost image lock while "
+                                << "re-registering image watch" << dendl;
+    } else if (r < 0) {
+      lderr(m_image_ctx.cct) << this
+                             << "failed to lock image while re-registering "
+                             << "image watch" << cpp_strerror(r) << dendl;
     }
   }
 
-  retry_aio_requests();
+  if (m_lock_owner_state == LOCK_OWNER_STATE_NOT_LOCKED) {
+    notify_listeners_updated_lock(LOCK_UPDATE_STATE_UNLOCKED);
+  }
 }
 
 void ImageWatcher::WatchCtx::handle_notify(uint64_t notify_id,
@@ -1142,4 +1217,25 @@ void ImageWatcher::RemoteContext::finish(int r) {
   m_image_watcher.schedule_async_complete(m_async_request_id, r);
 }
 
+void ImageWatcher::notify_listeners_updated_lock(
+    LockUpdateState lock_update_state) {
+  assert(m_image_ctx.owner_lock.is_locked());
+
+  Listeners listeners;
+  {
+    Mutex::Locker listeners_locker(m_listeners_lock);
+    m_listeners_in_use = true;
+    listeners = m_listeners;
+  }
+
+  for (Listeners::iterator it = listeners.begin();
+       it != listeners.end(); ++it) {
+    (*it)->handle_lock_updated(lock_update_state);
+  }
+
+  Mutex::Locker listeners_locker(m_listeners_lock);
+  m_listeners_in_use = false;
+  m_listeners_cond.Signal();
+}
+
 }
