@@ -14,21 +14,26 @@
 
 
 #include <errno.h>
+#include <fcntl.h>
 
 #include <boost/spirit/include/qi.hpp>
 #include <boost/spirit/include/phoenix_operator.hpp>
 #include <boost/spirit/include/phoenix.hpp>
 
+#include "common/debug.h"
 #include "MDSAuthCaps.h"
 
+#define dout_subsys ceph_subsys_mds
+
+#undef dout_prefix
+#define dout_prefix *_dout << "MDSAuthCap "
+
 using std::ostream;
 using std::string;
 namespace qi = boost::spirit::qi;
 namespace ascii = boost::spirit::ascii;
 namespace phoenix = boost::phoenix;
 
-const std::string MDSCapMatch::MDS_AUTH_PATH_ROOT = "/";
-
 template <typename Iterator>
 struct MDSCapParser : qi::grammar<Iterator, MDSAuthCaps()>
 {
@@ -36,6 +41,7 @@ struct MDSCapParser : qi::grammar<Iterator, MDSAuthCaps()>
   {
     using qi::char_;
     using qi::int_;
+    using qi::uint_;
     using qi::lexeme;
     using qi::alnum;
     using qi::_val;
@@ -52,12 +58,14 @@ struct MDSCapParser : qi::grammar<Iterator, MDSAuthCaps()>
       lexeme[lit("'") >> *(char_ - '\'') >> '\''];
     unquoted_path %= +char_("a-zA-Z0-9_.-/");
 
-    // match := [path=<path>] [uid=<uid>]
-    uid %= (spaces >> lit("uid") >> lit('=') >> int_);
+    // match := [path=<path>] [uid=<uid> [gids=<gid>[,<gid>...]]
     path %= (spaces >> lit("path") >> lit('=') >> (quoted_path | unquoted_path));
+    uid %= (spaces >> lit("uid") >> lit('=') >> uint_);
+    uintlist %= (uint_ % lit(','));
+    gidlist %= -(spaces >> lit("gids") >> lit('=') >> uintlist);
     match = -(
-             (uid)[_val = phoenix::construct<MDSCapMatch>(_1)] |
-             (path >> uid)[_val = phoenix::construct<MDSCapMatch>(_1, _2)] | 
+	     (uid >> gidlist)[_val = phoenix::construct<MDSCapMatch>(_1, _2)] |
+	     (path >> uid >> gidlist)[_val = phoenix::construct<MDSCapMatch>(_1, _2, _3)] |
              (path)[_val = phoenix::construct<MDSCapMatch>(_1)]);
 
     // capspec = * | r[w]
@@ -77,13 +85,49 @@ struct MDSCapParser : qi::grammar<Iterator, MDSAuthCaps()>
   qi::rule<Iterator, string()> quoted_path, unquoted_path;
   qi::rule<Iterator, MDSCapSpec()> capspec;
   qi::rule<Iterator, string()> path;
-  qi::rule<Iterator, int()> uid;
+  qi::rule<Iterator, uint32_t()> uid;
+  qi::rule<Iterator, std::vector<uint32_t>() > uintlist;
+  qi::rule<Iterator, std::vector<uint32_t>() > gidlist;
   qi::rule<Iterator, MDSCapMatch()> match;
   qi::rule<Iterator, MDSCapGrant()> grant;
   qi::rule<Iterator, std::vector<MDSCapGrant>()> grants;
   qi::rule<Iterator, MDSAuthCaps()> mdscaps;
 };
 
+void MDSCapMatch::normalize_path()
+{
+  // drop any leading /
+  while (path.length() && path[0] == '/') {
+    path = path.substr(1);
+  }
+
+  // drop dup //
+  // drop .
+  // drop ..
+}
+
+bool MDSCapMatch::match(const std::string &target_path,
+			const int caller_uid,
+			const int caller_gid) const
+{
+  if (uid != MDS_AUTH_UID_ANY) {
+    if (uid != caller_uid)
+      return false;
+    if (std::find(gids.begin(), gids.end(), caller_gid) == gids.end())
+      return false;
+  }
+  if (path.length()) {
+    if (target_path.find(path) != 0)
+      return false;
+    // if path doesn't already have a trailing /, make sure the target
+    // does so that path=/foo doesn't match target_path=/food
+    if (target_path.length() > path.length() &&
+	path[path.length()-1] != '/' &&
+	target_path[path.length()] != '/')
+      return false;
+  }
+  return true;
+}
 
 /**
  * For a given filesystem path, query whether this capability carries`
@@ -91,18 +135,71 @@ struct MDSCapParser : qi::grammar<Iterator, MDSAuthCaps()>
  *
  * This is true if any of the 'grant' clauses in the capability match the
  * requested path + op.
- *
  */
-bool MDSAuthCaps::is_capable(const std::string &path, int uid, bool may_read, bool may_write) const
+bool MDSAuthCaps::is_capable(const std::string &inode_path,
+			     uid_t inode_uid, gid_t inode_gid,
+			     unsigned inode_mode,
+			     uid_t caller_uid, gid_t caller_gid,
+			     unsigned mask,
+			     uid_t new_uid, gid_t new_gid) const
 {
-  for (std::vector<MDSCapGrant>::const_iterator i = grants.begin(); i != grants.end(); ++i) {
-    if (i->match.match(path, uid)) {
-      if ((may_read && !i->spec.read) ||
-          (may_write && !i->spec.write)) {
-        continue;
-      } else {
+  if (cct)
+    ldout(cct, 10) << __func__ << " inode(path /" << inode_path
+		   << " owner " << inode_uid << ":" << inode_gid
+		   << " mode 0" << std::oct << inode_mode << std::dec
+		   << ") by caller " << caller_uid << ":" << caller_gid
+		   << " mask " << mask
+		   << " new " << new_uid << ":" << new_gid
+		   << " cap: " << *this << dendl;
+
+  for (std::vector<MDSCapGrant>::const_iterator i = grants.begin();
+       i != grants.end();
+       ++i) {
+
+    if (i->match.match(inode_path, caller_uid, caller_gid) &&
+	i->spec.allows(mask & (MAY_READ|MAY_EXECUTE), mask & MAY_WRITE)) {
+
+      // check unix permissions?
+      if (i->match.uid == MDSCapMatch::MDS_AUTH_UID_ANY) {
         return true;
       }
+
+      // chown/chgrp
+      if (mask & MAY_CHOWN) {
+	if (new_uid != caller_uid ||   // you can't chown to someone else
+	    inode_uid != caller_uid) { // you can't chown from someone else
+	  continue;
+	}
+      }
+      if (mask & MAY_CHGRP) {
+	// you can only chgrp *to* one of your groups... if you own the file.
+	if (inode_uid != caller_uid ||
+	    std::find(i->match.gids.begin(), i->match.gids.end(), new_gid) ==
+	    i->match.gids.end()) {
+	  continue;
+	}
+      }
+
+      if (inode_uid == caller_uid) {
+        if ((!(mask & MAY_READ) || (inode_mode & S_IRUSR)) &&
+	    (!(mask & MAY_WRITE) || (inode_mode & S_IWUSR)) &&
+	    (!(mask & MAY_EXECUTE) || (inode_mode & S_IXUSR))) {
+          return true;
+        }
+      } else if (std::find(i->match.gids.begin(), i->match.gids.end(),
+			   inode_gid) != i->match.gids.end()) {
+        if ((!(mask & MAY_READ) || (inode_mode & S_IRGRP)) &&
+	    (!(mask & MAY_WRITE) || (inode_mode & S_IWGRP)) &&
+	    (!(mask & MAY_EXECUTE) || (inode_mode & S_IXGRP))) {
+          return true;
+        }
+      } else {
+        if ((!(mask & MAY_READ) || (inode_mode & S_IROTH)) &&
+	    (!(mask & MAY_WRITE) || (inode_mode & S_IWOTH)) &&
+	    (!(mask & MAY_EXECUTE) || (inode_mode & S_IXOTH))) {
+          return true;
+        }
+      }
     }
   }
 
@@ -115,7 +212,7 @@ void MDSAuthCaps::set_allow_all()
     grants.push_back(MDSCapGrant(MDSCapSpec(true, true, true), MDSCapMatch()));
 }
 
-bool MDSAuthCaps::parse(const std::string& str, ostream *err)
+bool MDSAuthCaps::parse(CephContext *c, const std::string& str, ostream *err)
 {
   // Special case for legacy caps
   if (str == "allow") {
@@ -129,6 +226,7 @@ bool MDSAuthCaps::parse(const std::string& str, ostream *err)
   std::string::const_iterator end = str.end();
 
   bool r = qi::phrase_parse(iter, end, g, ascii::space, *this);
+  cct = c;  // set after parser self-assignment
   if (r && iter == end) {
     return true;
   } else {
@@ -136,7 +234,7 @@ bool MDSAuthCaps::parse(const std::string& str, ostream *err)
     grants.clear();
 
     if (err)
-      *err << "osdcap parse failed, stopped at '" << std::string(iter, end)
+      *err << "MDSAuthCaps parse failed, stopped at '" << std::string(iter, end)
            << "' of '" << str << "'\n";
     return false; 
   }
@@ -157,14 +255,24 @@ bool MDSAuthCaps::allow_all() const
 
 ostream &operator<<(ostream &out, const MDSCapMatch &match)
 {
-  if (match.path != MDSCapMatch::MDS_AUTH_PATH_ROOT) {
-    out << "path=\"" << match.path << "\"";
-  }
-  if (match.path != MDSCapMatch::MDS_AUTH_PATH_ROOT && match.uid != MDSCapMatch::MDS_AUTH_UID_ANY) {
-    out << " ";
+  if (match.path.length()) {
+    out << "path=\"/" << match.path << "\"";
+    if (match.uid != MDSCapMatch::MDS_AUTH_UID_ANY) {
+      out << " ";
+    }
   }
   if (match.uid != MDSCapMatch::MDS_AUTH_UID_ANY) {
     out << "uid=" << match.uid;
+    if (!match.gids.empty()) {
+      out << " gids=";
+      for (std::vector<gid_t>::const_iterator p = match.gids.begin();
+	   p != match.gids.end();
+	   ++p) {
+	if (p != match.gids.begin())
+	  out << ',';
+	out << *p;
+      }
+    }
   }
 
   return out;
