@@ -236,6 +236,12 @@ void MDSDaemon::set_up_admin_socket()
                                      asok_hook,
                                      "scrub an inode and output results");
   assert(r == 0);
+  r = admin_socket->register_command("tag path",
+                                     "tag path name=path,type=CephString"
+                                     " name=tag,type=CephString",
+                                     asok_hook,
+                                     "Apply scrub tag recursively");
+   assert(r == 0);
   r = admin_socket->register_command("flush_path",
                                      "flush_path name=path,type=CephString",
                                      asok_hook,
@@ -328,6 +334,7 @@ const char** MDSDaemon::get_tracked_conf_keys() const
   static const char* KEYS[] = {
     "mds_op_complaint_time", "mds_op_log_threshold",
     "mds_op_history_size", "mds_op_history_duration",
+    "mds_enable_op_tracker",
     // clog & admin clog
     "clog_to_monitors",
     "clog_to_syslog",
@@ -357,6 +364,11 @@ void MDSDaemon::handle_conf_change(const struct md_config_t *conf,
                                                conf->mds_op_history_duration);
     }
   }
+  if (changed.count("mds_enable_op_tracker")) {
+    if (mds_rank) {
+      mds_rank->op_tracker.set_tracking(conf->mds_enable_op_tracker);
+    }
+  }
   if (changed.count("clog_to_monitors") ||
       changed.count("clog_to_syslog") ||
       changed.count("clog_to_syslog_level") ||
