@@ -37,9 +37,9 @@ public:
   bufferlist rollback;
 
   EFragment() : LogEvent(EVENT_FRAGMENT) { }
-  EFragment(MDLog *mdlog, int o, inodeno_t i, frag_t bf, int b) : 
+  EFragment(MDLog *mdlog, int o, dirfrag_t df, int b) :
     LogEvent(EVENT_FRAGMENT), metablob(mdlog), 
-    op(o), ino(i), basefrag(bf), bits(b) { }
+    op(o), ino(df.ino), basefrag(df.frag), bits(b) { }
 
   void print(ostream& out) const {
     out << "EFragment " << op_name(op) << " " << ino << " " << basefrag << " by " << bits << " " << metablob;
@@ -68,11 +68,13 @@ public:
       ::encode(*drb, rollback);
   }
 
+  EMetaBlob *get_metablob() { return &metablob; }
+
   void encode(bufferlist &bl) const;
   void decode(bufferlist::iterator &bl);
   void dump(Formatter *f) const;
   static void generate_test_instances(list<EFragment*>& ls);
-  void replay(MDS *mds);
+  void replay(MDSRank *mds);
 };
 
 #endif
