@@ -30,11 +30,11 @@ struct HealthService : public QuorumService
   HealthService(Monitor *m) : QuorumService(m) { }
   virtual ~HealthService() { }
 
-  virtual bool service_dispatch(Message *m) {
-    return service_dispatch(static_cast<MMonHealth*>(m));
+  virtual bool service_dispatch(MonOpRequestRef op) {
+    return service_dispatch_op(op);
   }
 
-  virtual bool service_dispatch(MMonHealth *m) = 0;
+  virtual bool service_dispatch_op(MonOpRequestRef op) = 0;
 
 public:
   virtual void get_health(Formatter *f,
