@@ -134,7 +134,7 @@ class DispatchQueue;
 
     SimpleMessenger *msgr;
     uint64_t conn_id;
-    ostream& _pipe_prefix(std::ostream *_dout);
+    ostream& _pipe_prefix(std::ostream &out) const;
 
     Pipe* get() {
       return static_cast<Pipe*>(RefCountedObject::get());
@@ -179,6 +179,11 @@ class DispatchQueue;
   private:
     int sd;
     struct iovec msgvec[IOV_MAX];
+#if !defined(MSG_NOSIGNAL) && !defined(SO_NOSIGPIPE)
+    sigset_t sigpipe_mask;
+    bool sigpipe_pending;
+    bool sigpipe_unblock;
+#endif
 
   public:
     int port;
@@ -230,7 +235,7 @@ class DispatchQueue;
 
     int read_message(Message **pm,
 		     AuthSessionHandler *session_security_copy);
-    int write_message(ceph_msg_header& h, ceph_msg_footer& f, bufferlist& body);
+    int write_message(const ceph_msg_header& h, const ceph_msg_footer& f, bufferlist& body);
     /**
      * Write the given data (of length len) to the Pipe's socket. This function
      * will loop until all passed data has been written out.
@@ -247,6 +252,10 @@ class DispatchQueue;
     int write_keepalive();
     int write_keepalive2(char tag, const utime_t &t);
 
+    void suppress_sigpipe();
+    void restore_sigpipe();
+
+
     void fault(bool reader=false);
 
     void was_session_reset();
