@@ -410,8 +410,11 @@ enum scrub_error_type PGBackend::be_compare_scrub_objects(
     if (error != CLEAN)
       errorstream << ", ";
     error = SHALLOW_ERROR;
+    bool known = auth.size == be_get_ondisk_size(auth_oi.size);
     errorstream << "size " << candidate.size
-		<< " != known size " << auth.size;
+		<< " != "
+                << (known ? "known" : "best guess")
+                << " size " << auth.size;
   }
   for (map<string,bufferptr>::const_iterator i = auth.attrs.begin();
        i != auth.attrs.end();
@@ -568,9 +571,10 @@ void PGBackend::be_compare_scrubmaps(
       be_select_auth_object(*k, maps, okseed, &auth_oi);
     list<pg_shard_t> auth_list;
     if (auth == maps.end()) {
+      dout(10) << __func__ << ": unable to find any auth object" << dendl;
       ++shallow_errors;
-      errorstream << pgid.pgid << " soid " << *k
-		  << ": failed to pick suitable auth object\n";
+      errorstream << pgid << " shard " << j->first
+		  << ": soid failed to pick suitable auth object\n";
       continue;
     }
     auth_list.push_back(auth->first);
