@@ -549,6 +549,7 @@ void RGWBucketInfo::dump(Formatter *f) const
   encode_json("quota", quota, f);
   encode_json("num_shards", num_shards, f);
   encode_json("bi_shard_hash_type", (uint32_t)bucket_index_shard_hash_type, f);
+  encode_json("requester_pays", requester_pays, f);
 }
 
 void RGWBucketInfo::decode_json(JSONObj *obj) {
@@ -564,6 +565,7 @@ void RGWBucketInfo::decode_json(JSONObj *obj) {
   uint32_t hash_type;
   JSONDecoder::decode_json("bi_shard_hash_type", hash_type, obj);
   bucket_index_shard_hash_type = (uint8_t)hash_type;
+  JSONDecoder::decode_json("requester_pays", requester_pays, obj);
 }
 
 void RGWObjEnt::dump(Formatter *f) const
@@ -605,6 +607,7 @@ void rgw_obj::dump(Formatter *f) const
   encode_json("ns", ns, f);
   encode_json("object", object, f);
   encode_json("instance", instance, f);
+  encode_json("orig_obj", orig_obj, f);
 }
 
 void RGWZoneParams::dump(Formatter *f) const
@@ -747,7 +750,6 @@ void RGWRegionMap::decode_json(JSONObj *obj)
 {
   JSONDecoder::decode_json("regions", regions, obj);
   JSONDecoder::decode_json("master_region", master_region, obj);
-  JSONDecoder::decode_json("bucket_quota", bucket_quota, obj);
   JSONDecoder::decode_json("user_quota", user_quota, obj);
 }
 
