@@ -13,6 +13,7 @@
 #include "cls/lock/cls_lock_client.h"
 #include "cls/lock/cls_lock_types.h"
 #include "librbd/AioCompletion.h"
+#include "librbd/AioImageRequestWQ.h"
 #include "librbd/internal.h"
 #include "librbd/ImageCtx.h"
 #include "librbd/ImageWatcher.h"
@@ -40,12 +41,48 @@ void register_test_image_watcher() {
 class TestImageWatcher : public TestFixture {
 public:
 
-  TestImageWatcher() : m_watch_ctx(NULL), m_aio_completion_restarts(0),
-		       m_expected_aio_restarts(0),
-		       m_callback_lock("m_callback_lock")
+  TestImageWatcher() : m_watch_ctx(NULL), m_callback_lock("m_callback_lock")
   {
   }
 
+  struct LockListener : public librbd::ImageWatcher::Listener {
+    Mutex lock;
+    Cond cond;
+    size_t releasing_lock_count;
+    size_t lock_updated_count;
+    bool lock_owner;
+
+    LockListener()
+      : lock("lock"), releasing_lock_count(0), lock_updated_count(0),
+        lock_owner(false) {
+    }
+
+    virtual bool handle_requested_lock() {
+      return true;
+    }
+    virtual void handle_lock_updated(
+        librbd::ImageWatcher::LockUpdateState state) {
+      Mutex::Locker locker(lock);
+      ++lock_updated_count;
+
+      switch (state) {
+      case librbd::ImageWatcher::LOCK_UPDATE_STATE_NOT_SUPPORTED:
+      case librbd::ImageWatcher::LOCK_UPDATE_STATE_UNLOCKED:
+      case librbd::ImageWatcher::LOCK_UPDATE_STATE_NOTIFICATION:
+        lock_owner = false;
+        break;
+      case librbd::ImageWatcher::LOCK_UPDATE_STATE_RELEASING:
+        lock_owner = false;
+        ++releasing_lock_count;
+        break;
+      case librbd::ImageWatcher::LOCK_UPDATE_STATE_LOCKED:
+        lock_owner = true;
+        break;
+      }
+      cond.Signal();
+    }
+  };
+
   class WatchCtx : public librados::WatchCtx2 {
   public:
     WatchCtx(TestImageWatcher &parent) : m_parent(parent), m_handle(0) {}
@@ -73,8 +110,10 @@ public:
 	DECODE_FINISH(iter);
 
         NotifyOp notify_op = static_cast<NotifyOp>(op);
+        /*
 	std::cout << "NOTIFY: " << notify_op << ", " << notify_id
 		  << ", " << cookie << ", " << notifier_id << std::endl;
+        */
 
 	Mutex::Locker l(m_parent.m_callback_lock);
         m_parent.m_notify_payloads[notify_op] = payload;
@@ -126,11 +165,39 @@ public:
     return 0;
   }
 
+  void register_lock_listener(librbd::ImageCtx &ictx) {
+    ictx.image_watcher->register_listener(&m_lock_listener);
+  }
+
   int register_image_watch(librbd::ImageCtx &ictx) {
     m_watch_ctx = new WatchCtx(*this);
     return m_watch_ctx->watch(ictx);
   }
 
+  bool wait_for_releasing_lock(librbd::ImageCtx &ictx) {
+    Mutex::Locker locker(m_lock_listener.lock);
+    while (m_lock_listener.releasing_lock_count == 0) {
+      if (m_lock_listener.cond.WaitInterval(ictx.cct, m_lock_listener.lock,
+                                            utime_t(10, 0)) != 0) {
+        return false;
+      }
+    }
+    m_lock_listener.releasing_lock_count = 0;
+    return true;
+  }
+
+  bool wait_for_lock_updated(librbd::ImageCtx &ictx) {
+    Mutex::Locker locker(m_lock_listener.lock);
+    while (m_lock_listener.lock_updated_count == 0) {
+      if (m_lock_listener.cond.WaitInterval(ictx.cct, m_lock_listener.lock,
+                                            utime_t(10, 0)) != 0) {
+        return false;
+      }
+    }
+    m_lock_listener.lock_updated_count = 0;
+    return true;
+  }
+
   bool wait_for_notifies(librbd::ImageCtx &ictx) {
     Mutex::Locker l(m_callback_lock);
     while (m_notifies.size() < m_notify_acks.size()) {
@@ -143,65 +210,6 @@ public:
     return (m_notifies.size() == m_notify_acks.size());
   }
 
-
-  librbd::AioCompletion *create_aio_completion(librbd::ImageCtx &ictx) {
-    librbd::AioCompletion *aio_completion = new librbd::AioCompletion();
-    aio_completion->complete_cb = &handle_aio_completion;
-    aio_completion->complete_arg = this;
-
-    aio_completion->init_time(&ictx, librbd::AIO_TYPE_NONE);
-    m_aio_completions.insert(aio_completion);
-    return aio_completion;
-  }
-
-  static void handle_aio_completion(void *arg1, void *arg2) {
-    TestImageWatcher *test_image_watcher =
-      reinterpret_cast<TestImageWatcher *>(arg2);
-    assert(test_image_watcher->m_callback_lock.is_locked());
-    test_image_watcher->m_callback_cond.Signal();
-  }
-
-  int handle_restart_aio(librbd::ImageCtx *ictx,
-			 librbd::AioCompletion *aio_completion) {
-    Mutex::Locker callback_locker(m_callback_lock);
-    ++m_aio_completion_restarts;
-
-    RWLock::RLocker owner_locker(ictx->owner_lock);
-    if (!ictx->image_watcher->is_lock_owner() &&
-        (m_expected_aio_restarts == 0 ||
-	 m_aio_completion_restarts < m_expected_aio_restarts)) {
-      ictx->image_watcher->request_lock(
-        boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-	aio_completion);
-    } else {
-      {
-	Mutex::Locker completion_locker(aio_completion->lock);
-	aio_completion->complete(ictx->cct);
-      }
-
-      m_aio_completions.erase(aio_completion);
-      aio_completion->release();
-    }
-
-    m_callback_cond.Signal();
-    return 0;
-  }
-
-  bool wait_for_aio_completions(librbd::ImageCtx &ictx) {
-    Mutex::Locker l(m_callback_lock);
-    int r = 0;
-    while (!m_aio_completions.empty() &&
-           (m_expected_aio_restarts == 0 ||
-	    m_aio_completion_restarts < m_expected_aio_restarts)) {
-      r = m_callback_cond.WaitInterval(ictx.cct, m_callback_lock,
-				       utime_t(10, 0));
-      if (r != 0) {
-        break;
-      }
-    }
-    return (r == 0);
-  }
-
   bufferlist create_response_message(int r) {
     bufferlist bl;
     ::encode(ResponseMessage(r), bl);
@@ -263,16 +271,14 @@ public:
 
   WatchCtx *m_watch_ctx;
 
+  LockListener m_lock_listener;
+
   NotifyOps m_notifies;
   NotifyOpPayloads m_notify_payloads;
   NotifyOpPayloads m_notify_acks;
 
   AsyncRequestId m_async_request_id;
 
-  std::set<librbd::AioCompletion *> m_aio_completions;
-  uint32_t m_aio_completion_restarts;
-  uint32_t m_expected_aio_restarts;
-
   Mutex m_callback_lock;
   Cond m_callback_cond;
 
@@ -476,17 +482,17 @@ TEST_F(TestImageWatcher, TryLockWithUserSharedLocked) {
   ASSERT_TRUE(ictx->image_watcher->is_lock_owner());
 }
 
-TEST_F(TestImageWatcher, UnlockNotLocked) {
+TEST_F(TestImageWatcher, ReleaseLockNotLocked) {
   REQUIRE_FEATURE(RBD_FEATURE_EXCLUSIVE_LOCK);
 
   librbd::ImageCtx *ictx;
   ASSERT_EQ(0, open_image(m_image_name, &ictx));
 
   RWLock::WLocker l(ictx->owner_lock);
-  ASSERT_EQ(0, ictx->image_watcher->unlock());
+  ASSERT_EQ(0, ictx->image_watcher->release_lock());
 }
 
-TEST_F(TestImageWatcher, UnlockNotifyReleaseLock) {
+TEST_F(TestImageWatcher, ReleaseLockNotifies) {
   REQUIRE_FEATURE(RBD_FEATURE_EXCLUSIVE_LOCK);
 
   librbd::ImageCtx *ictx;
@@ -504,7 +510,7 @@ TEST_F(TestImageWatcher, UnlockNotifyReleaseLock) {
   m_notify_acks += std::make_pair(NOTIFY_OP_RELEASED_LOCK, bufferlist());
   {
     RWLock::WLocker l(ictx->owner_lock);
-    ASSERT_EQ(0, ictx->image_watcher->unlock());
+    ASSERT_EQ(0, ictx->image_watcher->release_lock());
   }
   ASSERT_TRUE(wait_for_notifies(*ictx));
 
@@ -513,7 +519,7 @@ TEST_F(TestImageWatcher, UnlockNotifyReleaseLock) {
   ASSERT_EQ(expected_notify_ops, m_notifies);
 }
 
-TEST_F(TestImageWatcher, UnlockBrokenLock) {
+TEST_F(TestImageWatcher, ReleaseLockBrokenLock) {
   REQUIRE_FEATURE(RBD_FEATURE_EXCLUSIVE_LOCK);
 
   librbd::ImageCtx *ictx;
@@ -534,28 +540,50 @@ TEST_F(TestImageWatcher, UnlockBrokenLock) {
 					    lockers.begin()->first.cookie,
 					    lockers.begin()->first.locker));
 
-  ASSERT_EQ(0, ictx->image_watcher->unlock());
+  ASSERT_EQ(0, ictx->image_watcher->release_lock());
 }
 
 TEST_F(TestImageWatcher, RequestLock) {
   REQUIRE_FEATURE(RBD_FEATURE_EXCLUSIVE_LOCK);
 
+  librbd::ImageCtx *ictx;
+  ASSERT_EQ(0, open_image(m_image_name, &ictx));
+  ASSERT_EQ(0, register_image_watch(*ictx));
+
+  register_lock_listener(*ictx);
+  m_notify_acks = {{NOTIFY_OP_ACQUIRED_LOCK, {}}};
+
+  {
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ictx->image_watcher->request_lock();
+  }
+
+  ASSERT_TRUE(wait_for_notifies(*ictx));
+  NotifyOps expected_notify_ops;
+  expected_notify_ops += NOTIFY_OP_ACQUIRED_LOCK;
+  ASSERT_EQ(expected_notify_ops, m_notifies);
+
+  {
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ASSERT_TRUE(ictx->image_watcher->is_lock_owner());
+  }
+}
+
+TEST_F(TestImageWatcher, RequestLockFromPeer) {
+  REQUIRE_FEATURE(RBD_FEATURE_EXCLUSIVE_LOCK);
+
   librbd::ImageCtx *ictx;
   ASSERT_EQ(0, open_image(m_image_name, &ictx));
   ASSERT_EQ(0, register_image_watch(*ictx));
   ASSERT_EQ(0, lock_image(*ictx, LOCK_EXCLUSIVE,
 			  "auto " + stringify(m_watch_ctx->get_handle())));
 
+  register_lock_listener(*ictx);
   m_notify_acks = {{NOTIFY_OP_REQUEST_LOCK, create_response_message(0)}};
 
   {
-    RWLock::WLocker l(ictx->owner_lock);
-    ictx->image_watcher->request_lock(
-      boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx));
-    ictx->image_watcher->request_lock(
-      boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx));
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ictx->image_watcher->request_lock();
   }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
@@ -565,8 +593,11 @@ TEST_F(TestImageWatcher, RequestLock) {
 
   ASSERT_EQ(0, unlock_image());
 
-  m_notifies.clear();
-  m_notify_acks = {{NOTIFY_OP_RELEASED_LOCK,{}}, {NOTIFY_OP_ACQUIRED_LOCK,{}}};
+  {
+    Mutex::Locker l(m_callback_lock);
+    m_notifies.clear();
+    m_notify_acks = {{NOTIFY_OP_RELEASED_LOCK,{}}};
+  }
 
   bufferlist bl;
   {
@@ -575,13 +606,28 @@ TEST_F(TestImageWatcher, RequestLock) {
     ENCODE_FINISH(bl);
   }
   ASSERT_EQ(0, m_ioctx.notify2(ictx->header_oid, bl, 5000, NULL));
+  ASSERT_TRUE(wait_for_lock_updated(*ictx));
+
+  {
+    Mutex::Locker l(m_callback_lock);
+    m_notifies.clear();
+    m_notify_acks = {{NOTIFY_OP_ACQUIRED_LOCK, {}}};
+  }
+
+  {
+    RWLock::RLocker owner_lock(ictx->owner_lock);
+    ictx->image_watcher->request_lock();
+  }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
   expected_notify_ops.clear();
-  expected_notify_ops += NOTIFY_OP_RELEASED_LOCK, NOTIFY_OP_ACQUIRED_LOCK;
+  expected_notify_ops += NOTIFY_OP_ACQUIRED_LOCK;
   ASSERT_EQ(expected_notify_ops, m_notifies);
 
-  ASSERT_TRUE(wait_for_aio_completions(*ictx));
+  {
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ASSERT_TRUE(ictx->image_watcher->is_lock_owner());
+  }
 }
 
 TEST_F(TestImageWatcher, RequestLockTimedOut) {
@@ -593,14 +639,12 @@ TEST_F(TestImageWatcher, RequestLockTimedOut) {
   ASSERT_EQ(0, lock_image(*ictx, LOCK_EXCLUSIVE,
 			  "auto " + stringify(m_watch_ctx->get_handle())));
 
+  register_lock_listener(*ictx);
   m_notify_acks = {{NOTIFY_OP_REQUEST_LOCK, {}}};
 
-  m_expected_aio_restarts = 1;
   {
-    RWLock::WLocker l(ictx->owner_lock);
-    ictx->image_watcher->request_lock(
-      boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx));
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ictx->image_watcher->request_lock();
   }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
@@ -608,7 +652,27 @@ TEST_F(TestImageWatcher, RequestLockTimedOut) {
   expected_notify_ops += NOTIFY_OP_REQUEST_LOCK;
   ASSERT_EQ(expected_notify_ops, m_notifies);
 
-  ASSERT_TRUE(wait_for_aio_completions(*ictx));
+  // should resend when empty ack returned
+  {
+    Mutex::Locker l(m_callback_lock);
+    m_notifies.clear();
+  }
+  ASSERT_TRUE(wait_for_notifies(*ictx));
+
+  {
+    Mutex::Locker l(m_callback_lock);
+    ASSERT_EQ(0, unlock_image());
+    m_notifies.clear();
+    m_notify_acks = {{NOTIFY_OP_ACQUIRED_LOCK, {}}};
+  }
+
+  ASSERT_TRUE(wait_for_notifies(*ictx));
+  ASSERT_TRUE(wait_for_lock_updated(*ictx));
+
+  {
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ASSERT_TRUE(ictx->image_watcher->is_lock_owner());
+  }
 }
 
 TEST_F(TestImageWatcher, RequestLockIgnored) {
@@ -620,6 +684,7 @@ TEST_F(TestImageWatcher, RequestLockIgnored) {
   ASSERT_EQ(0, lock_image(*ictx, LOCK_EXCLUSIVE,
 			  "auto " + stringify(m_watch_ctx->get_handle())));
 
+  register_lock_listener(*ictx);
   m_notify_acks = {{NOTIFY_OP_REQUEST_LOCK, create_response_message(0)}};
 
   int orig_notify_timeout = ictx->cct->_conf->client_notify_timeout;
@@ -630,10 +695,8 @@ TEST_F(TestImageWatcher, RequestLockIgnored) {
   } BOOST_SCOPE_EXIT_END;
 
   {
-    RWLock::WLocker l(ictx->owner_lock);
-    ictx->image_watcher->request_lock(
-      boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx));
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ictx->image_watcher->request_lock();
   }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
@@ -642,11 +705,27 @@ TEST_F(TestImageWatcher, RequestLockIgnored) {
   ASSERT_EQ(expected_notify_ops, m_notifies);
 
   // after the request times out -- it will be resent
+  {
+    Mutex::Locker l(m_callback_lock);
+    m_notifies.clear();
+  }
   ASSERT_TRUE(wait_for_notifies(*ictx));
   ASSERT_EQ(expected_notify_ops, m_notifies);
 
-  ASSERT_EQ(0, unlock_image());
-  ASSERT_TRUE(wait_for_aio_completions(*ictx));
+  {
+    Mutex::Locker l(m_callback_lock);
+    ASSERT_EQ(0, unlock_image());
+    m_notifies.clear();
+    m_notify_acks = {{NOTIFY_OP_ACQUIRED_LOCK, {}}};
+  }
+
+  ASSERT_TRUE(wait_for_notifies(*ictx));
+  ASSERT_TRUE(wait_for_lock_updated(*ictx));
+
+  {
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ASSERT_TRUE(ictx->image_watcher->is_lock_owner());
+  }
 }
 
 TEST_F(TestImageWatcher, RequestLockTryLockRace) {
@@ -658,14 +737,12 @@ TEST_F(TestImageWatcher, RequestLockTryLockRace) {
   ASSERT_EQ(0, lock_image(*ictx, LOCK_EXCLUSIVE,
                           "auto " + stringify(m_watch_ctx->get_handle())));
 
+  register_lock_listener(*ictx);
   m_notify_acks = {{NOTIFY_OP_REQUEST_LOCK, create_response_message(0)}};
 
-  m_expected_aio_restarts = 1;
   {
-    RWLock::WLocker l(ictx->owner_lock);
-    ictx->image_watcher->request_lock(
-      boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx));
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ictx->image_watcher->request_lock();
   }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
@@ -673,8 +750,11 @@ TEST_F(TestImageWatcher, RequestLockTryLockRace) {
   expected_notify_ops += NOTIFY_OP_REQUEST_LOCK;
   ASSERT_EQ(expected_notify_ops, m_notifies);
 
-  m_notifies.clear();
-  m_notify_acks = {{NOTIFY_OP_RELEASED_LOCK, {}}};
+  {
+    Mutex::Locker l(m_callback_lock);
+    m_notifies.clear();
+    m_notify_acks = {{NOTIFY_OP_RELEASED_LOCK, {}}};
+  }
 
   bufferlist bl;
   {
@@ -683,45 +763,59 @@ TEST_F(TestImageWatcher, RequestLockTryLockRace) {
     ENCODE_FINISH(bl);
   }
   ASSERT_EQ(0, m_ioctx.notify2(ictx->header_oid, bl, 5000, NULL));
-  ASSERT_TRUE(wait_for_aio_completions(*ictx));
-  RWLock::RLocker l(ictx->owner_lock);
-  ASSERT_FALSE(ictx->image_watcher->is_lock_owner());
-}
 
-TEST_F(TestImageWatcher, RequestLockPreTryLockFailed) {
-  REQUIRE_FEATURE(RBD_FEATURE_EXCLUSIVE_LOCK);
+  // after losing race -- it will re-request
+  ASSERT_TRUE(wait_for_notifies(*ictx));
 
-  librbd::ImageCtx *ictx;
-  ASSERT_EQ(0, open_image(m_image_name, &ictx));
-  ASSERT_EQ(0, lock_image(*ictx, LOCK_SHARED, "manually 1234"));
-
-  m_expected_aio_restarts = 1;
   {
-    RWLock::WLocker l(ictx->owner_lock);
-    ictx->image_watcher->request_lock(
-      boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx));
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ASSERT_FALSE(ictx->image_watcher->is_lock_owner());
+  }
+
+  {
+    Mutex::Locker l(m_callback_lock);
+    ASSERT_EQ(0, unlock_image());
+    m_notifies.clear();
+    m_notify_acks = {{NOTIFY_OP_RELEASED_LOCK, {}}};
+  }
+
+  ASSERT_EQ(0, m_ioctx.notify2(ictx->header_oid, bl, 5000, NULL));
+  ASSERT_TRUE(wait_for_lock_updated(*ictx));
+
+  {
+    Mutex::Locker l(m_callback_lock);
+    m_notifies.clear();
+    m_notify_acks = {{NOTIFY_OP_ACQUIRED_LOCK, {}}};
+  }
+
+  {
+    RWLock::RLocker owner_lock(ictx->owner_lock);
+    ictx->image_watcher->request_lock();
+  }
+
+  ASSERT_TRUE(wait_for_lock_updated(*ictx));
+  ASSERT_TRUE(wait_for_notifies(*ictx));
+
+  {
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ASSERT_TRUE(ictx->image_watcher->is_lock_owner());
   }
-  ASSERT_TRUE(wait_for_aio_completions(*ictx));
 }
 
-TEST_F(TestImageWatcher, RequestLockPostTryLockFailed) {
+TEST_F(TestImageWatcher, RequestLockTryLockFailed) {
   REQUIRE_FEATURE(RBD_FEATURE_EXCLUSIVE_LOCK);
 
   librbd::ImageCtx *ictx;
   ASSERT_EQ(0, open_image(m_image_name, &ictx));
   ASSERT_EQ(0, register_image_watch(*ictx));
-  ASSERT_EQ(0, lock_image(*ictx, LOCK_EXCLUSIVE,
-                          "auto " + stringify(m_watch_ctx->get_handle())));
+  ASSERT_EQ(0, lock_image(*ictx, LOCK_SHARED, "manually 1234"));
 
-  m_notify_acks = {{NOTIFY_OP_REQUEST_LOCK, create_response_message(0)}};
+  register_lock_listener(*ictx);
+  m_notify_acks = {{NOTIFY_OP_REQUEST_LOCK, {}}};
 
-  m_expected_aio_restarts = 1;
   {
-    RWLock::WLocker l(ictx->owner_lock);
-    ictx->image_watcher->request_lock(
-      boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx));
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ictx->image_watcher->request_lock();
   }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
@@ -729,20 +823,21 @@ TEST_F(TestImageWatcher, RequestLockPostTryLockFailed) {
   expected_notify_ops += NOTIFY_OP_REQUEST_LOCK;
   ASSERT_EQ(expected_notify_ops, m_notifies);
 
-  ASSERT_EQ(0, unlock_image());
-  ASSERT_EQ(0, lock_image(*ictx, LOCK_SHARED, "manually 1234"));
-
-  m_notifies.clear();
-  m_notify_acks = {{NOTIFY_OP_RELEASED_LOCK, bufferlist()}};
-
-  bufferlist bl;
+  // should resend when error encountered
   {
-    ENCODE_START(1, 1, bl);
-    ::encode(NOTIFY_OP_RELEASED_LOCK, bl);
-    ENCODE_FINISH(bl);
+    Mutex::Locker l(m_callback_lock);
+    m_notifies.clear();
   }
-  ASSERT_EQ(0, m_ioctx.notify2(ictx->header_oid, bl, 5000, NULL));
-  ASSERT_TRUE(wait_for_aio_completions(*ictx));
+  ASSERT_TRUE(wait_for_notifies(*ictx));
+
+  {
+    Mutex::Locker l(m_callback_lock);
+    ASSERT_EQ(0, unlock_image());
+    m_notifies.clear();
+    m_notify_acks = {{NOTIFY_OP_ACQUIRED_LOCK, {}}};
+  }
+
+  ASSERT_TRUE(wait_for_notifies(*ictx));
 }
 
 TEST_F(TestImageWatcher, NotifyHeaderUpdate) {
@@ -1060,3 +1155,48 @@ TEST_F(TestImageWatcher, NotifyAsyncRequestTimedOut) {
   ASSERT_TRUE(thread.timed_join(boost::posix_time::seconds(10)));
   ASSERT_EQ(-ERESTART, flatten_task.result);
 }
+
+TEST_F(TestImageWatcher, PeerRequestsLock) {
+  REQUIRE_FEATURE(RBD_FEATURE_EXCLUSIVE_LOCK);
+
+  librbd::ImageCtx *ictx;
+  ASSERT_EQ(0, open_image(m_image_name, &ictx));
+  ASSERT_EQ(0, register_image_watch(*ictx));
+
+  register_lock_listener(*ictx);
+  m_notify_acks = {{NOTIFY_OP_ACQUIRED_LOCK, {}}};
+
+  {
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ictx->image_watcher->request_lock();
+  }
+
+  ASSERT_TRUE(wait_for_notifies(*ictx));
+
+  {
+    RWLock::RLocker owner_locker(ictx->owner_lock);
+    ASSERT_TRUE(ictx->image_watcher->is_lock_owner());
+  }
+
+  // if journaling is enabled, ensure we wait for it to replay since
+  // it will block our peer request
+  std::string buffer(256, '1');
+  ictx->aio_work_queue->write(0, buffer.size(), buffer.c_str(), 0);
+
+  {
+    Mutex::Locker l(m_callback_lock);
+    m_notifies.clear();
+    m_notify_acks = {{NOTIFY_OP_RELEASED_LOCK, {}}};
+  }
+
+  bufferlist bl;
+  {
+    ENCODE_START(1, 1, bl);
+    ::encode(NOTIFY_OP_REQUEST_LOCK, bl);
+    ENCODE_FINISH(bl);
+  }
+  ASSERT_EQ(0, m_ioctx.notify2(ictx->header_oid, bl, 5000, NULL));
+
+  ASSERT_TRUE(wait_for_releasing_lock(*ictx));
+  ASSERT_TRUE(wait_for_notifies(*ictx));
+}
