@@ -11,7 +11,7 @@ extern "C"{
 #define S3_BUCKET_NAME "s3testgw.fcgi"
 #define SWIFT_BUCKET_NAME "swift3testgw.fcgi"
 #define BUCKET_URL \
-  ((g_test->get_key_type() == KEY_TYPE_S3)?(string("/"S3_BUCKET_NAME)):(string("/swift/v1/"SWIFT_BUCKET_NAME)))
+  ((g_test->get_key_type() == KEY_TYPE_S3)?(string("/" S3_BUCKET_NAME)):(string("/swift/v1/" SWIFT_BUCKET_NAME)))
 #define GTEST
 #ifdef GTEST
 #include <gtest/gtest.h>
@@ -40,7 +40,8 @@ using namespace std;
 extern "C" int ceph_armor(char *dst, const char *dst_end, 
                           const char *src, const char *end);
 enum key_type {
-  KEY_TYPE_SWIFT = 1,
+  KEY_TYPE_UNDEFINED = 0,
+  KEY_TYPE_SWIFT,
   KEY_TYPE_S3
 };
 
@@ -65,7 +66,7 @@ class test_cors_helper {
     unsigned resp_code;
     key_type kt;
   public:
-    test_cors_helper() : resp_data(NULL){
+    test_cors_helper() : curl_inst(NULL), resp_data(NULL), resp_code(0), kt(KEY_TYPE_UNDEFINED){
       curl_global_init(CURL_GLOBAL_ALL);
     }
     ~test_cors_helper(){
@@ -247,8 +248,11 @@ int test_cors_helper::send_request(string method, string res,
       string s3auth;
       if(get_s3_auth(method, creds, date, res, s3auth) < 0)return -1;
       auth.append(string("Authorization: AWS ") + s3auth);
-    }else if(kt == KEY_TYPE_SWIFT){
+    } else if(kt == KEY_TYPE_SWIFT){
       auth.append(string("X-Auth-Token: ") + creds);
+    } else {
+      cout << "Unknown state (" << kt << ")\n";
+      return -1;
     }
 
     struct curl_slist *slist = NULL;
@@ -281,13 +285,13 @@ Finisher *finisher;
 
 static int create_bucket(void){
   if(g_test->get_key_type() == KEY_TYPE_S3){
-    g_test->send_request(string("PUT"), string("/"S3_BUCKET_NAME));
+    g_test->send_request(string("PUT"), string("/" S3_BUCKET_NAME));
     if(g_test->get_resp_code() != 200U){
       cout << "Error creating bucket, http code " << g_test->get_resp_code();
       return -1;
     }
   }else if(g_test->get_key_type() == KEY_TYPE_SWIFT){
-    g_test->send_request(string("PUT"), string("/swift/v1/"SWIFT_BUCKET_NAME));
+    g_test->send_request(string("PUT"), string("/swift/v1/" SWIFT_BUCKET_NAME));
     if(g_test->get_resp_code() != 201U){
       cout << "Error creating bucket, http code " << g_test->get_resp_code();
       return -1;
@@ -298,13 +302,13 @@ static int create_bucket(void){
 
 static int delete_bucket(void){
   if(g_test->get_key_type() == KEY_TYPE_S3){
-    g_test->send_request(string("DELETE"), string("/"S3_BUCKET_NAME));
+    g_test->send_request(string("DELETE"), string("/" S3_BUCKET_NAME));
     if(g_test->get_resp_code() != 204U){
       cout << "Error deleting bucket, http code " << g_test->get_resp_code();
       return -1;
     }
   }else if(g_test->get_key_type() == KEY_TYPE_SWIFT){
-    g_test->send_request(string("DELETE"), string("/swift/v1/"SWIFT_BUCKET_NAME));
+    g_test->send_request(string("DELETE"), string("/swift/v1/" SWIFT_BUCKET_NAME));
     if(g_test->get_resp_code() != 204U){
       cout << "Error deleting bucket, http code " << g_test->get_resp_code();
       return -1;
@@ -353,7 +357,7 @@ void send_cors(set<string> o, set<string> h,
     s3 = static_cast<RGWCORSConfiguration_S3 *>(&config);
     s3->to_xml(ss);
 
-    g_test->send_request(string("PUT"), string("/"S3_BUCKET_NAME"?cors"), cors_read_xml, 
+    g_test->send_request(string("PUT"), string("/" S3_BUCKET_NAME "?cors"), cors_read_xml, 
                          (void *)&ss, ss.str().length());
   }else if(g_test->get_key_type() == KEY_TYPE_SWIFT){
     set<string>::iterator it;
@@ -388,7 +392,7 @@ void send_cors(set<string> o, set<string> h,
     //const char *data = "1";
     stringstream ss;
     ss << "1";
-    g_test->send_request(string("POST"), string("/swift/v1/"SWIFT_BUCKET_NAME), cors_read_xml, 
+    g_test->send_request(string("POST"), string("/swift/v1/" SWIFT_BUCKET_NAME), cors_read_xml, 
                          (void *)&ss, 1);
   }
 }
@@ -396,7 +400,7 @@ void send_cors(set<string> o, set<string> h,
 TEST(TestCORS, getcors_firsttime){
   if(g_test->get_key_type() == KEY_TYPE_SWIFT)return;
   ASSERT_EQ(0, create_bucket());
-  g_test->send_request(string("GET"), string("/"S3_BUCKET_NAME"?cors"));
+  g_test->send_request(string("GET"), string("/" S3_BUCKET_NAME "?cors"));
   EXPECT_EQ(404U, g_test->get_resp_code());
   ASSERT_EQ(0, delete_bucket());
 }
@@ -414,7 +418,7 @@ TEST(TestCORS, putcors_firsttime){
 
   /*Now get the CORS and check if its fine*/
   if(g_test->get_key_type() == KEY_TYPE_S3){
-    g_test->send_request(string("GET"), string("/"S3_BUCKET_NAME"?cors"));
+    g_test->send_request(string("GET"), string("/" S3_BUCKET_NAME "?cors"));
     EXPECT_EQ(200U, g_test->get_resp_code());
 
     RGWCORSRule *r = xml_to_cors_rule(string("example.com"));
@@ -850,7 +854,7 @@ TEST(TestCORS, optionscors_test_options_7){
 TEST(TestCORS, deletecors_firsttime){
   if(g_test->get_key_type() == KEY_TYPE_SWIFT)return;
   ASSERT_EQ(0, create_bucket());
-  g_test->send_request("DELETE", "/"S3_BUCKET_NAME"?cors");
+  g_test->send_request("DELETE", "/" S3_BUCKET_NAME "?cors");
   EXPECT_EQ(204U, g_test->get_resp_code());
   ASSERT_EQ(0, delete_bucket());
 }
@@ -866,11 +870,11 @@ TEST(TestCORS, deletecors_test){
   send_cors(origins, h, e, flags, CORS_MAX_AGE_INVALID);
   EXPECT_EQ(((g_test->get_key_type() == KEY_TYPE_SWIFT)?202U:200U), g_test->get_resp_code());
 
-  g_test->send_request("GET", "/"S3_BUCKET_NAME"?cors");
+  g_test->send_request("GET", "/" S3_BUCKET_NAME "?cors");
   EXPECT_EQ(200U, g_test->get_resp_code());
-  g_test->send_request("DELETE", "/"S3_BUCKET_NAME"?cors");
+  g_test->send_request("DELETE", "/" S3_BUCKET_NAME "?cors");
   EXPECT_EQ(204U, g_test->get_resp_code());
-  g_test->send_request("GET", "/"S3_BUCKET_NAME"?cors");
+  g_test->send_request("GET", "/" S3_BUCKET_NAME "?cors");
   EXPECT_EQ(404U, g_test->get_resp_code());
   ASSERT_EQ(0, delete_bucket());
 }
