@@ -396,52 +396,6 @@ void dump_log(Formatter *formatter, ostream &out, pg_log_t &log,
   formatter->flush(out);
 }
 
-//Based on RemoveWQ::_process()
-void remove_coll(ObjectStore *store, const coll_t &coll,
-		 ObjectStore::Sequencer &osr)
-{
-  spg_t pg;
-  coll.is_pg_prefix(&pg);
-  OSDriver driver(
-    store,
-    coll_t(),
-    OSD::make_snapmapper_oid());
-  SnapMapper mapper(&driver, 0, 0, 0, pg.shard);
-
-  ghobject_t next;
-  int r = 0;
-  int64_t num = 0;
-  ObjectStore::Transaction t;
-  cout << "remove_coll " << coll << std::endl;
-  while (!next.is_max()) {
-    vector<ghobject_t> objects;
-    r = store->collection_list(coll, next, ghobject_t::get_max(), true, 300,
-      &objects, &next);
-    if (r < 0)
-      return;
-    for (vector<ghobject_t>::iterator i = objects.begin();
-	 i != objects.end();
-	 ++i, ++num) {
-
-      OSDriver::OSTransaction _t(driver.get_transaction(&t));
-      cout << "remove " << *i << std::endl;
-      int r = mapper.remove_oid(i->hobj, &_t);
-      if (r != 0 && r != -ENOENT) {
-        assert(0);
-      }
-
-      t.remove(coll, *i);
-      if (num >= 30) {
-        store->apply_transaction(&osr, t);
-        t = ObjectStore::Transaction();
-        num = 0;
-      }
-    }
-  }
-  t.remove_collection(coll);
-  store->apply_transaction(&osr, t);
-}
-
 //Based on part of OSD::load_pgs()
 int finish_remove_pgs(ObjectStore *store)
 {
@@ -746,6 +700,8 @@ int set_inc_osdmap(ObjectStore *store, epoch_t e, bufferlist& bl, bool force,
     }
     cout << "Creating a new epoch." << std::endl;
   }
+  if (dry_run)
+    return 0;
   ObjectStore::Transaction t;
   t.write(coll_t::meta(), inc_oid, 0, bl.length(), bl);
   t.truncate(coll_t::meta(), inc_oid, bl.length());
@@ -791,6 +747,8 @@ int set_osdmap(ObjectStore *store, epoch_t e, bufferlist& bl, bool force,
     }
     cout << "Creating a new epoch." << std::endl;
   }
+  if (dry_run)
+    return 0;
   ObjectStore::Transaction t;
   t.write(coll_t::meta(), full_oid, 0, bl.length(), bl);
   t.truncate(coll_t::meta(), full_oid, bl.length());
@@ -1689,6 +1647,9 @@ int do_set_attr(ObjectStore *store, coll_t coll,
   if (ret < 0)
     return ret;
 
+  if (dry_run)
+    return 0;
+
   t->touch(coll, ghobj);
 
   t->setattr(coll, ghobj, key,  bl);
@@ -1707,6 +1668,9 @@ int do_rm_attr(ObjectStore *store, coll_t coll,
   if (debug)
     cerr << "Rmattr " << ghobj << std::endl;
 
+  if (dry_run)
+    return 0;
+
   t->rmattr(coll, ghobj, key);
 
   store->apply_transaction(&osr, *t);
@@ -1762,6 +1726,9 @@ int do_set_omap(ObjectStore *store, coll_t coll,
 
   attrset.insert(pair<string, bufferlist>(key, valbl));
 
+  if (dry_run)
+    return 0;
+
   t->touch(coll, ghobj);
 
   t->omap_setkeys(coll, ghobj, attrset);
@@ -1783,6 +1750,9 @@ int do_rm_omap(ObjectStore *store, coll_t coll,
   if (debug)
     cerr << "Rm_omap " << ghobj << std::endl;
 
+  if (dry_run)
+    return 0;
+
   t->omap_rmkeys(coll, ghobj, keys);
 
   store->apply_transaction(&osr, *t);
@@ -1824,6 +1794,9 @@ int do_set_omaphdr(ObjectStore *store, coll_t coll,
   if (ret)
     return ret;
 
+  if (dry_run)
+    return 0;
+
   t->touch(coll, ghobj);
 
   t->omap_setheader(coll, ghobj, hdrbl);
@@ -2175,6 +2148,9 @@ int remove_clone(ObjectStore *store, coll_t coll, ghobject_t &ghobj, snapid_t cl
   ret = remove_from(snapset.clone_size, "clone_size", cloneid, force);
   if (ret) return ret;
 
+  if (dry_run)
+    return 0;
+
   bufferlist bl;
   ::encode(snapset, bl);
   ObjectStore::Transaction t;
@@ -3234,6 +3210,10 @@ int main(int argc, char **argv)
       cout << "Remove past-intervals " << past_intervals << std::endl;
 
       past_intervals.clear();
+      if (dry_run) {
+        ret = 0;
+        goto out;
+      }
       ret = write_info(*t, map_epoch, info, past_intervals);
 
       if (ret == 0) {
@@ -3261,11 +3241,13 @@ int main(int argc, char **argv)
       info.history.last_epoch_clean = superblock.current_epoch;
       past_intervals.clear();
 
-      ret = write_info(*t, map_epoch, info, past_intervals);
-      if (ret == 0) {
+      if (!dry_run) {
+	ret = write_info(*t, map_epoch, info, past_intervals);
+	if (ret != 0)
+	  goto out;
 	fs->apply_transaction(osr, *t);
-	cout << "Marking complete succeeded" << std::endl;
       }
+      cout << "Marking complete succeeded" << std::endl;
     } else {
       assert(!"Should have already checked for valid --op");
     }
